import java.util.ArrayList;
import greenfoot.*;
import packages.twoD.Point;
import packages.twoD.Vector;

public class Game extends World
{
    private static final int SPEED = 30; //The number of frames a move takes
    public static final int SCALE = 20; //The size of one tile
    private int turn; //The index of the player that is turn, or higher if the animation is running
    private double movePart;
    public ArrayList<Player> players;
    private GreenfootImage image;
    private GreenfootImage track;
    private String emulatedKey;
    
    public static String lastKey;

    public Game(){    
        super(600, 400, 1);
        setup();
    }

    private void setup(){
        init();
        setActOrder(Game.class);
        track = new GreenfootImage("images/track1.png");
        image = new GreenfootImage(track);
        image = drawSquares(image);
        setBackground(image);
        image = getBackground();
        updateBackground();
        players.add(new Player(0));
        players.add(new Player(1));
        for(int i=0; i<players.size(); i++){
            addObject(players.get(i), (10 + i) * SCALE, 2 * SCALE);
        }
        turn = -1;
        nextTurn();
        movePart = 0;
    }

    private void init(){
        players = new ArrayList<Player>();
    }

    public void act(){
        lastKey = Greenfoot.getKey();
        if(turn==players.size()){
            movePart++;
            if(movePart>SPEED){
                movePart = 0;
                nextTurn();
            }
        }
        else{
            movePart = 0;
            Player now = players.get(turn);
            try {
                switch(Integer.parseInt(emulatedKey)){
                    case 1:
                    now.changeSpeed(new Vector(-1, 1));
                    break;
                    case 2:
                    now.changeSpeed(new Vector(0, 1));
                    break;
                    case 3:
                    now.changeSpeed(new Vector(1, 1));
                    break;
                    case 4:
                    now.changeSpeed(new Vector(-1, 0));
                    break;
                    case 5:
                    now.changeSpeed(new Vector(0, 0));
                    break;
                    case 6:
                    now.changeSpeed(new Vector(1, 0));
                    break;
                    case 7:
                    now.changeSpeed(new Vector(-1, -1));
                    break;
                    case 8:
                    now.changeSpeed(new Vector(0, -1));
                    break;
                    case 9:
                    now.changeSpeed(new Vector(1, -1));
                    break;
                    default:
                    throw new Exception();
                }
                nextTurn();
            }catch (Exception e){}
            emulatedKey = null;
        }
        updateBackground();
    }

    private void nextTurn(){
        turn++;
        removeObjects(getObjects(Selector.class));
        if(turn>players.size()){
            turn = 0;
            for(Player p : players)p.finishMove();
        }
        if(turn<players.size()){
            switch(players.get(turn).getSurface()){
                /*case 1:
                    break;*/
                case 2:{
                    Point p = new Point(players.get(turn).getLocation()).add(players.get(turn).getSpeed().scale(SCALE));
                    Player c = players.get(turn);
                    addObject(new Selector(c, 2), (int)p.X, (int)(p.Y + SCALE));
                    addObject(new Selector(c, 4), (int)(p.X - SCALE), (int)p.Y);
                    addObject(new Selector(c, 5), (int)p.X, (int)p.Y);
                    addObject(new Selector(c, 6), (int)(p.X + SCALE), (int)p.Y);
                    addObject(new Selector(c, 8), (int)p.X, (int)(p.Y - SCALE));
                    break;
                }
                default:{
                    Point p = new Point(players.get(turn).getLocation()).add(players.get(turn).getSpeed().scale(SCALE));
                    Player c = players.get(turn);
                    addObject(new Selector(c, 1), (int)(p.X - SCALE), (int)(p.Y + SCALE));
                    addObject(new Selector(c, 2), (int)p.X, (int)(p.Y + SCALE));
                    addObject(new Selector(c, 3), (int)(p.X + SCALE), (int)(p.Y + SCALE));
                    addObject(new Selector(c, 4), (int)(p.X - SCALE), (int)p.Y);
                    addObject(new Selector(c, 5), (int)p.X, (int)p.Y);
                    addObject(new Selector(c, 6), (int)(p.X + SCALE), (int)p.Y);
                    addObject(new Selector(c, 7), (int)(p.X - SCALE), (int)(p.Y - SCALE));
                    addObject(new Selector(c, 8), (int)p.X, (int)(p.Y - SCALE));
                    addObject(new Selector(c, 9), (int)(p.X + SCALE), (int)(p.Y - SCALE));
                }
            }
        }
    }
    
    public int getSurfaceAt(int x, int y){
        try{
            Color c = track.getColorAt(x, y);
            if(c.equals(new Color(127, 127, 127)))return 1; //Wall
            else if(c.equals(Color.YELLOW))return 2; //Sand
            else if(c.equals(new Color(100, 100, 100)))return 3; //Slippery area
            return 0;
        }catch(Exception e){return 0;}
    }

    public double getMovePart(){
        return (double)movePart * (double)SCALE / (double)SPEED;
    }
    
    private void updateBackground(){
        setBackground(new GreenfootImage(image));
    }
    
    public void drawPreview(packages.twoD.Point p, int x, int y, Color c){
        getBackground().setColor(c);
        getBackground().drawLine((int)p.X, (int)p.Y, x, y);
    }
    public void drawHistory(packages.twoD.Point p, int x, int y, Color c){
        drawPreview(p, x, y, c);
        image.setColor(c);
        image.drawLine((int)p.X, (int)p.Y, x, y);
    }
    public void drawPoint(int x, int y, Color c){
        getBackground().setColor(c);
        getBackground().fillRect(x-1, y-1, 3, 3);
        image.setColor(c);
        image.fillRect(x-1, y-1, 3, 3);
    }

    public void emulate(String key){
        emulatedKey = key;
    }



    private static GreenfootImage drawSquares(GreenfootImage image){
        GreenfootImage square = new GreenfootImage(SCALE, SCALE);
        square.setColor(new Color(80, 80, 80, 100));
        square.drawRect(0, 0, SCALE, SCALE);
        for(int x=0; x<image.getWidth(); x+=SCALE){
            for(int y=0; y<image.getHeight(); y+=SCALE){
                image.drawImage(square, x, y);
            }
        }
        return image;
    }
}