import greenfoot.*;
import java.util.ArrayList;
import packages.twoD.*;
public class Player extends SmoothMover{
    public final static int SIZE = 10;
    
    private Vector speed;
    private Point location;
    private Point lastEdge;
    private int damage;
    private Color color;
    private ArrayList<Point> collidPoints;


    public Player(Color c){
        init();
        color = c;
        updateImage();
    }
    public Player(int n){
        init();
        switch(n){
            case 1:
            color = Color.GREEN;
            break;
            case 2:
            color = Color.BLUE;
            break;
            case 3:
            color = Color.YELLOW;
            break;
            default:
            color = Color.RED;
        }
        updateImage();
    }
    private void init(){
        speed = new Vector();
        collidPoints = new ArrayList(1);
    }
    private void updateImage(){
        GreenfootImage image = new GreenfootImage(SIZE, SIZE);
        image.setColor(color);
        image.setColor(color);
        image.fillOval(0, 0, image.getWidth() - 1, image.getHeight() - 1);
        setImage(image);
    }
    protected void addedToWorld(World w){
        location = new Point(getX(), getY());
        lastEdge = new Point(location);
    }

    public void changeSpeed(Vector v){
        speed.add(v);
    }
    public void setSpeed(Vector v){
        speed = new Vector(v);
    }
    public Vector getSpeed(){
        return new Vector(speed);
    }

    public void setPoint(Point p){
        location = new Point(p);
    }

    public void act(){
        step(((Game)getWorld()).getMovePart());
        ((Game)getWorld()).drawPreview(lastEdge, getX(), getY(), color);
    }


    private void step(double n){
        if(!goTo(location.X + speed.getX() * n, location.Y + speed.getY() * n)){
            damage((int)(speed.abs()));
            speed = new Vector();
            return;
        }
        Player colliding = (Player)getOneIntersectingObject(Player.class);
        if(n!=0&&colliding!=null&&!alreadyCollided()
        &&colliding.getExactX()==getExactX()&&colliding.getExactY()==getExactY()){
            collidPoints.add(new Point(getX(), getY()));
            colliding.collidPoints.add(new Point(getExactX(), getExactY()));
            
            ((Game)getWorld()).drawHistory(lastEdge, getX(), getY(), color);
            ((Game)getWorld()).drawHistory(colliding.lastEdge, colliding.getX(), colliding.getY(), colliding.color);
            lastEdge = new Point(getX(), getY());
            colliding.lastEdge = new Point(colliding.getX(), colliding.getY());
            
            damage(colliding);
            
            Point temp = new Point(colliding.location);
            colliding.location = new Point(location);
            location = temp;
            
            Vector temp1 = new Vector(colliding.speed);
            colliding.speed = new Vector(speed);
            speed = temp1;
        }
        if(n==1)collidPoints.clear();
    }
    
    public void finishMove(){
        ((Game)getWorld()).drawHistory(lastEdge, getX(), getY(), color);
        location = new Point(getX(), getY());
        lastEdge = new Point(location);
        ((Game)getWorld()).drawPoint(getX(), getY(), getColor());
    }

    private boolean goTo(double x, double y){
        Point now = new Point(getX(), getY());
        Vector move = new Vector(x - now.X, y - now.Y);
        for(int i=1; i<move.abs(); i++){
            setLocation(now.X + move.getX() * i / move.abs(), (now.Y + move.getY() * i / move.abs()));
            if(colliding())return false;
        }
        setLocation(x, y);
        if(colliding())return false;
        return true;
    }

    private boolean colliding(){
        return ((Game)getWorld()).getSurfaceAt(getX(), getY())==1;
    }
    
    private boolean alreadyCollided(){
        for(Point p : collidPoints){
            if(p.equals(new Point(getExactX(), getExactY())))return true;
        }
        return false;
    }

    private int damage(Player colliding){
        int damage = (int)(new Vector(speed.getPoint(), colliding.speed.getPoint()).abs());
        damage(damage);
        colliding.damage(damage);
        return damage;
    }

    public void damage(int amount){
        damage+=amount;
    }
    
    public void move(Vector v){
        setLocation((getX() + v.getX()), (getY() + v.getY()));
    }
    
    public Color getColor(){
        return color;
    }
    public Point getLocation(){
        return location;
    }

    public int getSurface(){
        return ((Game)getWorld()).getSurfaceAt(getX(), getY());
    }
}